package com.jiayue.passback.job;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.jiayue.passback.entity.ForecastPowerShortTerm;
import com.jiayue.passback.entity.eunm.PredictionModelEnum;
import com.jiayue.passback.mapper.ForecastPowerShortTermMapper;
import com.jiayue.passback.service.ForecastPowerShortTermService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;


/**
 * 解析修正短期数据定时任务
 */
@Component
@Slf4j
public class analysisJob {

    private final ForecastPowerShortTermMapper forecastPowerShortTermMapper;

    private final ForecastPowerShortTermService forecastPowerShortTermService;


    public analysisJob(ForecastPowerShortTermMapper forecastPowerShortTermMapper, ForecastPowerShortTermService forecastPowerShortTermService) {
        this.forecastPowerShortTermMapper = forecastPowerShortTermMapper;
        this.forecastPowerShortTermService = forecastPowerShortTermService;
    }

    @Scheduled(cron = "0 0/1 * * * ?")
    public void analysisShortTerm() {
        log.info("-----------------------解析修正短期数据定时任务开始执行-----------------------");
        String dayStr = new SimpleDateFormat("yyyyMMdd").format(new Date());

        // 扫描路径
        String scanPath = "/home/syjy/backFile" + File.separator;

        String newPath = scanPath + "new" + File.separator;

        String bak = scanPath + "bak" +File.separator +  dayStr + File.separator;

        String error = scanPath + "error" + File.separator;


        File bakFile = new File(bak);

        File errorFile = new File(error);

        File newFile = new File(newPath);


        // 判断目录是否存在
        if (!bakFile.exists()) {
            bakFile.mkdirs();
            log.info("【" + bakFile.getPath() + "】目录不存在，系统自动创建文件目录");
        }

        if (!errorFile.exists()) {
            errorFile.mkdirs();
            log.info("【" + errorFile.getPath() + "】目录不存在，系统自动创建文件目录");
        }

        if (!newFile.exists()) {
            newFile.mkdirs();
            log.info("【" + newFile.getPath() + "】目录不存在，系统自动创建文件目录");
        }

        log.info("系统扫描路径【" + newFile.getPath() + "】");

        Collection<File> files = FileUtils.listFiles(newFile, new String[]{"CIME"}, false);


        if (files.size() > 0) {
            for (File file : files) {
                if (!file.getName().contains(dayStr)) {
                    file.delete();
                    log.warn(file.getName() + "不是当天文件，系统自动删除！");
                    break;
                }
                log.info("开始解析文件【" + file.getName() + "】");
                List<ForecastPowerShortTerm> forecastPowerShortTermList = fileAnalysisShortTerm(file, new Date().getTime());
                if (null != forecastPowerShortTermList && forecastPowerShortTermList.size() > 0) {
                    this.updateShortTerm(forecastPowerShortTermList);
                    log.info("短期数据更新成功！文件记录数" + forecastPowerShortTermList.size() + "条");
                    try {
                        FileUtils.moveFile(new File(newFile + File.separator + file.getName()), new File(bakFile + File.separator + file.getName()));
                        log.info("文件【" + file.getName() + "】移动到bak目录成功！");
                    } catch (IOException e) {
                        throw new RuntimeException(e);
                    }
                } else {
                    try {
                        FileUtils.moveFile(new File(newFile + File.separator + file.getName()), new File(errorFile + File.separator + file.getName()));
                        log.warn("文件内容为空，不进行数据更新！移动到失败目录下");
                    } catch (IOException e) {
                        throw new RuntimeException(e);
                    }
                }
            }
        }

    }


    /**
     * 更新数据
     *
     * @param shortTermList
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public void updateShortTerm(List<ForecastPowerShortTerm> shortTermList) {
        // 获取明天开始时间和结束时间
        long startTime = shortTermList.get(0).getForecastTime();
        long endTime = shortTermList.get(shortTermList.size() - 1).getForecastTime();
        // 根据开始时间和结束时间作为删除的查询条件
        LambdaQueryWrapper<ForecastPowerShortTerm> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.between(ForecastPowerShortTerm::getForecastTime, startTime, endTime);
        shortTermList.sort(Comparator.comparing(ForecastPowerShortTerm::getForecastTime));
        // 删除明天的预测数据
        forecastPowerShortTermMapper.delete(lambdaQueryWrapper);
        // 保存明天的预测数据
        forecastPowerShortTermService.saveBatch(shortTermList);
    }


    private List<ForecastPowerShortTerm> fileAnalysisShortTerm(File file, Long currentDate) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmm");
        List<ForecastPowerShortTerm> forecastPowerShortTerm = new ArrayList<>();
        // 当文件未被使用时，进行解析上报
        if (file.renameTo(file)) {
            InputStreamReader read = null;
            BufferedReader bufferedReader = null;
            String stringLine;
            ForecastPowerShortTerm stf;
            try {
                read = new InputStreamReader(Files.newInputStream(file.toPath()), StandardCharsets.UTF_8);
                bufferedReader = new BufferedReader(read);
                // 循环解析文件
                while ((stringLine = bufferedReader.readLine()) != null) {
                    String[] string_arr = stringLine.split(" ");
                    if (string_arr.length == 12 && string_arr[0].startsWith("#")) {
                        if (StringUtils.isNotEmpty(string_arr[2])) {
                            stf = new ForecastPowerShortTerm();
                            stf.setFpValue(new BigDecimal(string_arr[3]));
                            stf.setForecastTime(sdf.parse(string_arr[2]).getTime());
                            stf.setGenDate(new Date(currentDate));
                            stf.setPredictionModelEnum(PredictionModelEnum.E14);
                            forecastPowerShortTerm.add(stf);
                        }
                    }
                }
            } catch (IOException | ParseException | RuntimeException e) {
                log.error("系统错误：", e);
                File destFile = new File(file.getPath().replaceFirst("new", "error"));
                if (destFile.exists()) {
                    destFile.delete();
                }
                try {
                    FileUtils.moveFile(file, destFile);
                } catch (IOException e1) {
                    log.error(file.getName() + "文件解析失败", e);
                }
            } finally {
                close(bufferedReader, read);
            }
        }
        return forecastPowerShortTerm;
    }

    private void close(BufferedReader bufferedReader, InputStreamReader read) {
        try {
            if (bufferedReader != null) {
                bufferedReader.close();
            }
            if (read != null) {
                read.close();
            }
        } catch (IOException e) {
            log.error("关闭文件流失败:", e);
        }
    }


//    private void moveFileBackups(String filePath) {
//        String targetRoot = filePath.replaceFirst("backupsTemp", "backups");
//        // 移动文件夹内容
//        File sourceFile = new File(filePath);
//        if (sourceFile.exists()) {
//            try {
//                File[] files = sourceFile.listFiles();
//                if (files != null) {
//                    for (File f : files) {
//                        if (f.renameTo(new File(path + f.getName()))) {
//                            log.info("move file :{}, dest file:{}", path, f.getName());
//                        }
//                    }
//                }
//            } catch (Exception e) {
//                log.error("文件移动失败" + e);
//            }
//        }
//    }

}
